"""
Tests for basic simulation model objects instantiation
"""

from __future__ import absolute_import
import unittest
from Epigrass.manager import *
from Epigrass.simobj import siteobj, graph, edge
from Epigrass.report import Report


class testObjInstantiation(unittest.TestCase):
    def setUp(self):
        self.curdir = os.getcwd()
        os.chdir('./demos/')
        self.sitios = loadData('sitios3.csv', sep=',')
        self.edges = loadData('edgesout.csv', sep=',')
        self.S = Simulate('flu.epg')

    def tearDown(self):
        os.chdir(self.curdir)
        if os.path.exists('demos/outdata-flu'):
            os.system('rm -rf demos/outdata-flu')

    def testSites(self):
        l = self.S.instSites(self.sitios)
        for i in range(len(l)):
            self.assertEqual(l[i].sitename, self.sitios[i][2])

    def testEdges(self):
        l = self.S.instSites(self.sitios)
        e = self.S.instEdges(l, self.edges)
        j = 0
        for i in e:
            self.assertEqual((i.source.geocode, i.dest.geocode), (int(self.edges[j][5]), int(self.edges[j][6])))
            j += 1

    def test_edge_migration(self):
        l = self.S.instSites(self.sitios)
        e = self.S.instEdges(l, self.edges)
        g = self.S.instGraph('grafo', 1, l, e)
        g.simstep = 100
        for edge in e:
            edge.migrate()
            self.assertGreater(len(edge.dest.thetalist), 0)

    def testGraph(self):
        l = self.S.instSites(self.sitios)
        e = self.S.instEdges(l, self.edges)
        g = self.S.instGraph('grafo', 1, l, e)
        self.assertEquals(len(g.site_dict), len(self.sitios))
        self.assertEquals(len(g.edge_dict), len(self.edges))

    def test_getAllPairs(self):
        l = self.S.instSites(self.sitios)
        e = self.S.instEdges(l, self.edges)
        g = self.S.instGraph('grafo', 1, l, e)
        pairs = g.getAllPairs()

    def test_report(self):
        R = Report(self.S)
        src = R.Assemble(3, False)
        self.assertIn('Full Report', src)


class TestSimulationRuns(unittest.TestCase):
    def setUp(self):
        self.curdir = os.getcwd()
        os.chdir('demos/')
        #        self.sitios3 = loadData('sitios3.csv',sep=',')
        #        self.nodes = loadData('nodes.csv',sep=',')
        #        self.edgesout = loadData('edgesout.csv',sep=',')
        pass

    def test_mesh_epg(self):
        S = Simulate('mesh.epg', silent=True)
        S.start()

    def test_backend_csv(self):
        S = Simulate('mesh.epg', silent=True)
        S.backend = 'csv'
        S.start()

    def test_custom_model(self):
        S = Simulate('sars.epg', silent=True)
        S.start()

    def test_custom_model_parallel(self):
        S = Simulate('sars.epg', silent=True)
        S.parallel = True
        S.start()

    def tearDown(self):
        if os.path.exists('outdata-mesh'):
            os.system('rm -rf outdata-mesh')
        if os.path.exists('outdata-sars'):
            os.system('rm -rf outdata-sars')
        os.chdir('..')


if __name__ == '__main__':
    unittest.main()



import unittest
from Epigrass.manager import Simulate
import geopandas as gpd
import os
import pathlib

class SimulateTestCase(unittest.TestCase):
    def test_Instantiate(self):
        os.chdir(pathlib.Path(__file__).parent.absolute())
        S = Simulate(fname='SEIR.epg', backend='sqlite')
        self.assertIsInstance(S, Simulate)
        self.assertIsInstance(S.World.nodes, gpd.GeoDataFrame)


if __name__ == '__main__':
    unittest.main()



import pandas as pd
import altair as alt
import networkx as NX
import nx_altair as nxa
import geopandas as gpd
import os
import panel as pn
import panel.widgets as pnw
import hvplot.pandas
import holoviews as hv
import geoviews as gv
from holoviews.operation.datashader import datashade, bundle_graph
from holoviews import opts
from bokeh.resources import INLINE
import param
import base64
from sqlalchemy import create_engine
import glob
from functools import lru_cache

enc_icon = b'iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAA3ElEQVRYhe2WWQ7DIAxEH1XvVR+dmzkfVSJngWIW0VYZKV8EZgzDmABARFkhBDxomQs8z+tFvfoxBUG8nHkBgnircAlOwlt5LzxmkN4CbgFfJeB950vSrDHxUihOwtbEKxaQScKxQTUrCU87YJE5jnEeOJJfkdkxNUcTaDCnrbb0OCJRFbavmtySer3QVcAMI/5GEmaN1utNqAIhpjwghm8/Lsg2twbTd8CsU2/AlrnTTbhDbSX/swPgr2ZIeHl6QStX8tqsi79MBqxXMNcpu+LY7Ub0i48VdOv3CSxJ9X3LgJP02QAAAABJRU5ErkJggg=='
with open('egicon.png', 'wb') as f:
    f.write(base64.b64decode(enc_icon))
material = pn.template.MaterialTemplate(title='Epigrass Dashboard', favicon='egicon.png', logo='egicon.png')

pn.config.sizing_mode = 'stretch_width'
alt.renderers.set_embed_options(actions=True)


@lru_cache(maxsize=10)
def get_sims(fname):
    """
    Get list of simulations available on SQLite database
    :param pth: Database file
    :return: List of tables. list of str
    """
    full_path = os.path.join(fname, 'Epigrass.sqlite')
    if os.path.exists(full_path):
        con = create_engine(f'sqlite:///{full_path}?check_same_thread=False').connect()
        sims = con.execute("SELECT name FROM sqlite_master WHERE type='table' ORDER BY name;")
        sim_names = [s[0] for s in sims if not (s[0].endswith('_meta') or s[0].endswith('e'))]
        con.close()
        return sim_names
    else:
        print(f'==> File {full_path} not found')
        get_sims.cache_clear()
        return []


def get_graph(pth):
    full_path = os.path.join(os.path.abspath(pth), 'network.gml')
    if os.path.exists(full_path):
        G = NX.read_gml(full_path)
    else:
        G = NX.MultiDiGraph()
    return G


@lru_cache(maxsize=10)
def get_meta_table(fname, simname):
    full_path = os.path.join(os.path.abspath(fname), 'Epigrass.sqlite')
    if os.path.exists(full_path):
        mname = simname + '_meta'
        con = create_engine(f'sqlite:///{full_path}?check_same_thread=False').connect()
        df = pd.read_sql_query(f"select * from {mname}", con)
        con.close()
    else:
        df = pd.DataFrame()
        get_meta_table.cache_clear()
    return df


@lru_cache(maxsize=10)
def read_simulation(fname, simulation_name, locality=None):
    full_path = os.path.join(os.path.abspath(fname), 'Epigrass.sqlite')
    if not os.path.exists(full_path):
        print(f'==> File {full_path} not found')
        read_simulation.cache_clear()
        return pd.DataFrame()

    con = create_engine(f'sqlite:///{full_path}?check_same_thread=False').connect()
    if locality is None:
        simdf = pd.read_sql_query(f"select * from {simulation_name}", con)
    else:
        simdf = pd.read_sql_query(f"select * from {simulation_name} where name='{locality}'", con)
    simdf.fillna(0, inplace=True)
    con.close()
    return simdf


@lru_cache(maxsize=10)
def get_localities(fname, simulation_name):
    full_path = os.path.join(os.path.abspath(fname), 'Epigrass.sqlite')
    if not os.path.exists(full_path):
        print(f'==> File {full_path} not found')
        get_localities.cache_clear()
        return []
    con = create_engine(f'sqlite:///{full_path}?check_same_thread=False').connect()
    locs = pd.read_sql_query(f'select distinct name from {simulation_name}', con)
    localities = [l[0] for l in locs.values]
    con.close()
    return localities


@lru_cache(maxsize=10)
def read_map(fname):
    if os.path.exists(fname):
        return gpd.read_file(fname)
    else:
        gpd.GeoDataFrame()
        read_map.cache_clear()


def get_subgraph(G, node):
    """
    return subgraph containing `node` and its direct neighboors
    :param G: full graph
    :param node: node defining the subgraph
    """
    nodes = [node]
    nodes.extend(list(G.neighbors(node)))
    H = G.subgraph(nodes).copy()
    return H


# pipeline = pn.pipeline.Pipeline()
class SeriesViewer(param.Parameterized):
    model_path = param.String(default='../demos/outdata-rio')
    map_selector = param.ObjectSelector()
    simulation_run = param.ObjectSelector()
    localities = param.ObjectSelector(default='Pick a Locality', objects=[])
    time_slider = param.Integer(default=1, bounds=(1, 300))

    def __init__(self, **params):
        self.update()
        self.update_sims()
        super(SeriesViewer, self).__init__(**params)

    @param.depends('model_path', watch=True)
    def update(self):
        maps = glob.glob(os.path.join(self.model_path, '*.gpkg'))
        self.param['map_selector'].objects = maps
        if maps:
            self.map_selector = maps[0]
        self.mapdf = read_map(os.path.join(self.model_path, 'Data.gpkg'))

    @param.depends('map_selector', 'model_path', watch=True)
    def update_sims(self):
        sims = get_sims(self.model_path)
        self.param['simulation_run'].objects = sims
        if sims:
            self.simulation_run = sims[-1]
        self.df = read_simulation(self.model_path, self.simulation_run)

    @param.depends('simulation_run', watch=True)
    def update_localities(self):
        locs = get_localities(self.model_path, self.simulation_run)
        self.param['localities'].objects = list(locs)
        if locs:
            self.localities = locs[0]

    @param.depends('map_selector', 'simulation_run')
    def view_meta(self):
        df = get_meta_table(self.model_path, self.simulation_run)
        if len(df) > 0:
            dfpars = df[[c for c in df.columns if 'parameters' in c]]
            dfpars.columns = [c.split('$')[-1] for c in dfpars.columns]
            return pn.pane.Markdown(
                f"""
## Simulation Info
### Map File
{df['the_world$shapefile'].iloc[0].split(',')[0].strip("['")}
### Sites File
{df['the_world$sites'].iloc[0]}
### Edges File
{df['the_world$sites'].iloc[0]}
### Model Type
{df['epidemiological_model$modtype'].iloc[0]}
### Epidemic Events
Seed: {df['epidemic_events$seed']}

"""
            )
        else:
            return pn.pane.Markdown("")

    @param.depends('model_path', 'map_selector', 'simulation_run')
    def view_map(self):
        if self.map_selector and self.mapdf is not None:
            mapdf = self.mapdf
            map10 = mapdf.sort_values('totalcases', ascending=False).iloc[:15]
            brush = alt.selection_single(encodings=["y"], on="mouseover", empty='none')
            color = alt.Color('totalcases', scale=alt.Scale(type='pow', exponent=0.4))
            f = alt.hconcat(
                alt.Chart(map10).mark_bar().encode(
                    x=alt.X('totalcases', scale=alt.Scale(nice=True)),
                    y=alt.Y('name', sort=alt.EncodingSortField(field='totalcases',
                                                               op='sum', order='descending')),
                    tooltip=['name', 'prevalence', 'totalcases'],
                    color=alt.condition(brush, alt.value('lightgray'), color)
                ).add_selection(
                    brush
                ).properties(
                    width=200,
                    height='container'
                ),
                alt.Chart(self.mapdf).mark_geoshape(
                ).encode(
                    color=alt.condition(
                        brush,
                        alt.value('lightgray'),
                        color,
                    ),
                    tooltip=['name', 'prevalence', 'totalcases', 'arrivals']
                ).properties(
                    width=600,
                    height='container'
                )
            )
            # f = self.mapdf[self.mapdf.totalcases>0].hvplot.polygons(geo=True, c='prevalence',
            #                                alpha=0.7,
            #                                colormap='BuPu',
            #                                responsive=True,
            #                                colorbar=True,
            #                                tiles=True,
            #                                )

            return f
        else:
            return pn.indicators.LoadingSpinner(value=True, width=100, height=100)

    @param.depends('model_path', 'localities')
    def view_network(self):
        if self.mapdf is None:
            return pn.indicators.LoadingSpinner(value=True, width=100, height=100)
        G = get_graph(self.model_path)
        if G.order == 0:
            return pn.pane.Alert(f'## No network file found on {self.model_path}')
        nodeloc = list(filter(lambda n: n[1]['name'] == self.localities, G.nodes(data=True)))
        # print(nodeloc, self.localities)
        if nodeloc == []:
            return pn.pane.Alert(f'## Please select a locality.')
        H = get_subgraph(G, nodeloc[0][0])
        partial_map = self.mapdf[self.mapdf.geocode.isin(H.nodes)]
        partial_map = partial_map.set_crs(4326)
        partial_map = partial_map.to_crs(3857)  # Converting to web mercator
        centroids = [(c.x, c.y) for c in partial_map.centroid]
        # Draw the graph using Altair
        gcs = [str(int(gc)) for gc in partial_map.geocode]
        pos = dict(zip(gcs, centroids))

        # viz = nxa.draw_networkx(
        #     G, pos=pos,
        #     node_color='weight',
        #     cmap='viridis',
        #     width='weight',
        #     edge_color='black',
        # )

        kwargs = dict(width=800, height=800, xaxis=None, yaxis=None)
        hv.opts.defaults(opts.Nodes(**kwargs), opts.Graph(**kwargs))
        colors = ['#000000'] + hv.Cycle('Category20').values
        epi_graph = hv.Graph.from_networkx(H, positions=pos)

        epi_graph.opts(
            cmap=colors, node_size=10, edge_line_width=1,
            directed=True,
            node_line_color='gray',
            edge_color='gray',
            node_color='circle'
        )
        tiles = gv.tile_sources.Wikipedia

        f = bundle_graph(epi_graph) * tiles
        source = epi_graph.nodes.clone()
        # print(epi_graph.nodes.data.iloc[0])
        source.data = epi_graph.nodes.data[epi_graph.nodes.data.name == self.localities]

        return f * source.opts(color='red')

    @param.depends('localities', 'time_slider', 'simulation_run')
    def view_series(self):
        if self.simulation_run is None:
            return pn.pane.Alert('## No data\nPlease select a simulation from the `Simulation run` widget on the left.')
        mapdf = self.mapdf
        df = self.df = read_simulation(self.model_path, self.simulation_run)
        # df = self.df[self.df.name==self.localities]
        if len(df) == 0:
            return pn.pane.Alert(f'## No data for "{self.localities}"\nPlease select a locality.')
        time = df.time.min()
        self.param['time_slider'].bounds = (df.time.min(), df.time.max())
        variables = [c for c in df.columns if c not in ['name', 'time', 'geocode', 'lat', 'longit']]
        if mapdf is None:
            return pn.indicators.LoadingSpinner(value=True, width=100, height=100)
        name_col = [c for c in mapdf if df.name[0] in list(mapdf[c])][0]
        mapa_t = pd.merge(mapdf, df[df.time == self.time_slider][['name', 'time'] + variables],
                          left_on=name_col, right_on='name')
        # series = df[df.name == self.localities].hvplot.line(
        #     width=400,
        #     x='time',
        #     y=variables,
        #     responsive=True,
        #     subplots=True,
        #     shared_axes=False,
        #     value_label=f'Cases at {self.localities}',
        # ).cols(3)
        mapa = alt.Chart(mapa_t).mark_geoshape(
        ).encode(
            color='Infectious',
            tooltip=['name'] + variables,
        ).properties(
            width='container',
            height='container'
        )
        # mapa = mapa_t[self.mapdf.totalcases > 0].hvplot.polygons(
        #     geo=True,
        #     hover_cols=variables,
        #     alpha=0.7,
        #     colormap='BuPu',
        #     responsive=True,
        #     title=f'State at time {self.time_slider}',
        #     c=variables[-2],
        #     colorbar=True,
        #     tiles=True,
        # )
        # f = (mapa + series).cols(1)

        return mapa

    @param.depends('localities', 'time_slider', 'simulation_run')
    def altair_series(self):
        if len(self.df) == 0:
            return
        df = self.df[self.df.name == self.localities]
        variables = [c for c in df.columns if c not in ['name', 'time', 'geocode', 'lat', 'longit']]

        base = alt.Chart(df).mark_line(interpolate='step-after').encode(
            x='time:Q',
            y='incidence:Q',
        ).properties(
            width='container',
            height=300
        )

        chart = alt.vconcat()
        for i, y_enc in enumerate(variables[::2]):
            row = alt.hconcat()
            row |= base.encode(x='time', y=y_enc)
            if i < len(variables):
                row |= base.encode(x='time', y=variables[i + 1])

            chart &= row
        return chart


def refresh(e):
    # Clear caches
    get_sims.cache_clear()
    get_localities.cache_clear()
    read_map.cache_clear()
    read_simulation.cache_clear()
    # Update the Panel
    series_viewer.update()
    series_viewer.update_sims()
    series_viewer.update_localities()


def main():
    global series_viewer
    series_viewer = SeriesViewer()  # model_path=sm.output()[0], sim_run=sm.output()[1], locality=sm.output()[2])
    button = pnw.Button(name='Refresh', button_type='primary')
    button.on_click(refresh)
    # Assembling the panel
    material.sidebar.append(pn.Param(series_viewer, name='Control Panel'))
    # material.sidebar.append(button)
    material.sidebar.append(pn.layout.Divider())
    material.sidebar.append(series_viewer.view_meta)
    # material.sidebar.append(pn.Row(save_fname, save_button))
    # material.sidebar.append(pn.widgets.StaticText(sm.simulation_run.path))
    material.main.append(
        pn.Column(
            pn.Row(
                pn.Card(series_viewer.view_map, title='Final State')
            ),
            pn.Row(
                pn.Card(series_viewer.view_network, title='Network')
            ),
            pn.Row(
                pn.Card(series_viewer.view_series, title='Time Map')
            ),
            pn.Row(
                pn.Card(series_viewer.altair_series, title='Time Series')
            ),

        )
    )
    return material, series_viewer


# material.servable();


def show(pth):
    material, series_viewer = main()
    series_viewer.model_path = pth
    pn.serve(material, port=5006)


if __name__ == "__main__":
    material, series_viewer = main()
    series_viewer.model_path = '../demos/outdata-rio'
    refresh(None)
    pn.serve(material, port=5006, threaded=False)



from __future__ import absolute_import
from __future__ import print_function
from six.moves import zip

__author__ = 'fccoelho'

import unittest
import numpy as np
# from Epigrass.manager import *
from Epigrass.simobj import siteobj, graph, edge
from Epigrass.epimodels import Epimodel


class TestModels(unittest.TestCase):
    def setUp(self):
        pass

    def test_run_SIS(self):
        model = Epimodel(1, modtype=b"SIS")
        res = model.step((0, 1, 999), 0, 1000, 0, 0, bi={'e': 0, 'i': 1, 's': 999},
                         bp={'r': 0.1, 'b': 0.01, 'beta': 0.01, 'alpha': 1.})
        for x, y in zip(res[0], [0, 0.9, 999.01]):
            self.assertAlmostEqual(x, y, 0)
        self.assertAlmostEqual(res[1], 0, 1)
        self.assertAlmostEqual(res[2], 0.9, 1)

        # TOdo: add asserts with the expected results of the models

    def test_run_SIS_s(self):
        model = Epimodel(1, modtype=b'SIS_s')
        res = model.step((0, 1, 999), 0, 1000, 0, 0, bi={'e': 0, 'i': 1, 's': 999},
                         bp={'r': 0.1, 'b': 0.01, 'beta': 0.01, 'alpha': 1.})
        for x, y in zip(res[0], [0, 0.9, 999.01]):
            self.assertAlmostEqual(x, y, 0)
        self.assertAlmostEqual(res[1], 0, 1)
        self.assertAlmostEqual(res[2], 0.9, 1)

    def test_run_SIR(self):
        model = Epimodel(1, modtype=b'SIR')
        res = model.step((0, 1, 999), 0, 1000, 0, 0, bi={b'e': 0, b'i': 1, b's': 999},
                         bp={b'r': 0.1, b'b': 0.01, b'beta': 0.01, b'alpha': 1.})

        for x, y in zip(res[0], [0, 0.9, 999.01]):
            self.assertAlmostEqual(x, y, 1)
        self.assertAlmostEqual(res[1], 0, 1)
        self.assertAlmostEqual(res[2], 0.9, 1)

    def test_run_SIR_s(self):
        model = Epimodel(1, modtype=b'SIR_s')
        res = model.step((0, 1, 999), 0, 1000, 0, 0, bi={'e': 0, 'i': 1, 's': 999},
                         bp={'r': 0.1, 'b': 0.01, 'beta': 0.01, 'alpha': 1.})

        for x, y in zip(res[0], [0, 0.9, 999.01]):
            self.assertAlmostEqual(x, y, 1)
        self.assertAlmostEqual(res[1], 0, 1)
        self.assertAlmostEqual(res[2], 0.9, 1)

    def test_run_SEIS(self):
        model = Epimodel(1, modtype=b'SEIS')
        res = model.step((0, 1, 999), 0, 1000, 0, 0, bi={'e': 0, 'i': 1, 's': 999},
                         bp={'r': 0.1, 'b': 0.01, 'e': 0.1, 'beta': 0.01, 'alpha': 1.})

        for x, y in zip(res[0], [0, 0.9, 999.01]):
            self.assertAlmostEqual(x, y, 0)
        self.assertAlmostEqual(res[1], 0, 1)
        self.assertAlmostEqual(res[2], 0.9, 1)

    def test_run_SEIS_s(self):
        model = Epimodel(1, modtype=b'SEIS_s')
        res = model.step((0, 1, 999), 0, 1000, 0, 0, bi={'e': 0, 'i': 1, 's': 999},
                         bp={'r': 0.1, 'b': 0.01, 'e': 0.1, 'beta': 0.01, 'alpha': 1.})

        for x, y in zip(res[0], [0, 0.9, 999.01]):
            self.assertAlmostEqual(x, y, 0)
        self.assertAlmostEqual(res[1], 0, 1)
        self.assertAlmostEqual(res[2], 0.9, 1)

    def test_run_SEIR(self):
        model = Epimodel(1, modtype=b'SEIR')
        res = model.step((0, 1, 999), 0, 1000, 0, 0, bi={'e': 0, 'i': 1, 's': 999},
                         bp={'r': 0.1, 'b': 0.01, 'e': 0.1, 'beta': 0.01, 'alpha': 1.})

        for x, y in zip(res[0], [0, 0.9, 999.01]):
            self.assertAlmostEqual(x, y, 0)
        self.assertAlmostEqual(res[1], 0, 1)
        self.assertAlmostEqual(res[2], 0.9, 1)

    def test_run_SEIR_s(self):
        model = Epimodel(1, modtype=b'SEIR_s')
        res = model.step((0, 1, 999), 0, 1000, 0, 0, bi={'e': 0, 'i': 1, 's': 999},
                         bp={'r': 0.1, 'b': 0.01, 'e': 0.1, 'beta': 0.01, 'alpha': 1.})

        for x, y in zip(res[0], [0, 0.9, 999.01]):
            self.assertAlmostEqual(x, y, 0)
        self.assertAlmostEqual(res[1], 0, 1)
        self.assertAlmostEqual(res[2], 0.9, 1)

    def test_run_SIpRpS(self):
        model = Epimodel(1, modtype=b'SIpRpS')
        res = model.step((0, 1, 999), 0, 1000, 0, 0, bi={'e': 0, 'i': 1, 's': 999},
                         bp={'r': 0.1, 'b': 0.01, 'beta': 0.01, 'alpha': 1., 'delta': 0.1})

        for x, y in zip(res[0], [0, 0.9, 999.01]):
            self.assertAlmostEqual(x, y, 0)
        self.assertAlmostEqual(res[1], 0, 1)
        self.assertAlmostEqual(res[2], 0.9, 1)

    # def test_run_Influenza(self):
    #     model = Epimodel(1, modtype=b'Influenza')
    #     res = model.step((999, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0), 0, 1000, 0, 0,
    #                      bi={b'e1': 0, b'is1': 1, b's1': 999, b'e2': 0, b'is2': 1, b's2': 0, b'e3': 0, b'is3': 1, b's3': 0,
    #                          b'e4': 0, b'is4': 1, b's4': 0, b'ic1': 0, b'ic2': 0, b'ic3': 0, b'ic4': 0, b'ig1': 0, b'ig2': 0,
    #                          b'ig3': 0, b'ig4': 0},
    #                      bp={b'r': 0.1, b'b': 0.01, b'beta': 0.01, b'e': 0.1, b'pc1': .1, b'c': 0.1, b'pp1': .1, b'g': .1,
    #                          b'd': .1, b'pc2': .1, b'pp2': .1, b'pc3': .1, b'pp3': .1, b'pc4': .1, b'pp4': .1, b'alpha': 1.,
    #                          b'delta': 0.1, b'vaccineNow': 1, b'vaccov': 0.3})
    #     # print res

    # for x, y in zip(res[0], [999.3065035, 0.0049949993999999996, 0.9, 0.010000000000000002, 0.0, 0.01, 0.0, 0.0, 0.0, 0.0, 0.01, 0.0, 0.0, 0.0, 0.0, 0.01, 0.0, 0.0, 0.0, 0.0]):
    #     self.assertAlmostEqual(x, y, 0)
    # self.assertAlmostEqual(res[1], 0.003496, 3)
    # self.assertAlmostEqual(res[2], 0.46, 2)

    def test_run_SIpRpS_s(self):
        model = Epimodel(1, modtype=b'SIpRpS_s')
        res = model.step((0, 1, 999), 0, 1000, 0, 0, bi={'e': 0, 'i': 1, 's': 999},
                         bp={'r': 0.1, 'b': 0.01, 'e': 0.1, 'beta': 0.01, 'alpha': 1., 'delta': 0.1})

        # print res
        for x, y in zip(res[0], [0, 0.9, 999.01]):
            self.assertAlmostEqual(x, y, 0)
        self.assertAlmostEqual(res[1], 0, 1)
        self.assertAlmostEqual(res[2], 0.9, 1)

    def test_run_SEIpRpS(self):
        model = Epimodel(1, modtype=b'SEIpRpS')
        res = model.step((0, 1, 999), 0, 1000, 0, 0, bi={'e': 0, 'i': 1, 's': 999},
                         bp={'r': 0.1, 'b': 0.01, 'e': 0.1, 'beta': 0.01, 'alpha': 1., 'delta': 0.1})

        for x, y in zip(res[0], [0, 0.9, 999.01]):
            self.assertAlmostEqual(x, y, 0)
        self.assertAlmostEqual(res[1], 0, 1)
        self.assertAlmostEqual(res[2], 0.9, 1)

    def test_run_SEIpRpS_s(self):
        model = Epimodel(1, modtype=b'SEIpRpS_s')
        res = model.step((0, 1, 999), 0, 1000, 0, 0, bi={'e': 0, 'i': 1, 's': 999},
                         bp={'r': 0.1, 'b': 0.01, 'e': 0.1, 'beta': 0.01, 'alpha': 1., 'delta': 0.1})

        for x, y in zip(res[0], [0, 0.9, 999.01]):
            self.assertAlmostEqual(x, y, 0)
        self.assertAlmostEqual(res[1], 0, 1)
        self.assertAlmostEqual(res[2], 0.9, 1)

    def test_run_SIpR(self):
        model = Epimodel(1, modtype=b'SIpR')
        res = model.step((0, 1, 999), 0, 1000, 0, 0, bi={'e': 0, 'i': 1, 's': 999},
                         bp={'r': 0.1, 'b': 0.01, 'beta': 0.01, 'alpha': 1., 'delta': 0.1, 'p': 0.1})

        for x, y in zip(res[0], [0, 0.9, 999.01]):
            self.assertAlmostEqual(x, y, 0)
        self.assertAlmostEqual(res[1], 0, 1)
        self.assertAlmostEqual(res[2], 0.9, 1)

    def test_run_SIpR_s(self):
        model = Epimodel(1, modtype=b'SIpR_s')
        res = model.step((0, 1, 999), 0, 1000, 0, 0, bi={'e': 0, 'i': 1, 's': 999},
                         bp={'r': 0.1, 'b': 0.01, 'beta': 0.01, 'alpha': 1., 'delta': 0.1, 'p': 0.1})

        for x, y in zip(res[0], [0, 0.9, 999]):
            self.assertAlmostEqual(x, y, 0)
        self.assertAlmostEqual(res[1], 0, 1)
        self.assertAlmostEqual(res[2], 0.9, 1)

    def test_run_SEIpR(self):
        model = Epimodel(1, modtype=b'SEIpR')
        res = model.step((0, 1, 999), 0, 1000, 0, 0, bi={'e': 0, 'i': 1, 's': 999},
                         bp={'r': 0.1, 'b': 0.01, 'e': 0.1, 'beta': 0.01, 'alpha': 1., 'delta': 0.1, 'p': 0.1})

        for x, y in zip(res[0], [0, 0.9, 999]):
            self.assertAlmostEqual(x, y, 0)
        self.assertAlmostEqual(res[1], 0, 1)
        self.assertAlmostEqual(res[2], 0.9, 1)

    def test_run_SEIpR_s(self):
        model = Epimodel(1, modtype=b'SEIpR_s')
        res = model.step((0, 10, 990), 0, 1000, 0, 0, bi={'e': 0, 'i': 1, 's': 999},
                         bp={'r': 0.1, 'b': 0.01, 'e': 0.1, 'beta': 0.01, 'alpha': 1., 'delta': 0.1, 'p': 0.1})

        for x, y in zip(res[0], [0, 9, 990]):
            self.assertAlmostEqual(x, y, 0)
        self.assertAlmostEqual(res[1], 0, 1)
        self.assertAlmostEqual(res[2], 9, 1)

    def test_run_SIRS(self):
        model = Epimodel(1, modtype=b'SIRS')
        res = model.step((0, 1, 999), 0, 1000, 0, 0, bi={'e': 0, 'i': 1, 's': 999},
                         bp={'r': 0.1, 'b': 0.01, 'beta': 0.01, 'alpha': 1., 'delta': 0.1, 'p': 0.1, 'w': 0.1})
        print(res)
        for x, y in zip(res[0], [0, 0.9, 999]):
            self.assertAlmostEqual(x, y, 0)
        self.assertAlmostEqual(res[1], 0, 1)
        self.assertAlmostEqual(res[2], 0.9, 1)

    def test_run_SIRS_s(self):
        model = Epimodel(1, modtype=b'SIRS_s')
        res = model.step((0, 1, 999), 0, 1000, 0, 0, bi={'e': 0, 'i': 1, 's': 999},
                         bp={'r': 0.1, 'b': 0.01, 'beta': 0.01, 'alpha': 1., 'delta': 0.1, 'p': 0.1, 'w': 0.1})
        for x, y in zip(res[0], [0, 0.9, 999.01]):
            self.assertAlmostEqual(x, y, 0)
        self.assertAlmostEqual(res[1], 0, 1)
        self.assertAlmostEqual(res[2], 0.9, 1)


import pylab as P


def run(model, args, n):
    res = []
    for i in range(n):
        args[1] = i
        out = model.step(*tuple(args))
        res.append(
            out[0]
        )
        args[0] = out[0]
    return np.array(res)


class test_model_run(unittest.TestCase):
    def setUp(self):
        self.bi = {'e': 0, 'i': 10, 's': 9999}
        self.bp = {'r': 0.1, 'b': 0.01, 'e': 0.1, 'beta': 1, 'alpha': 1., 'delta': 0.1, 'p': 0.1, 'w': 0.1}

    def tearDown(self):
        P.show()

    def test_run_SIRS_s(self):
        model = Epimodel(1, modtype=b'SIRS_s')
        res = run(model, [(0, 10, 999), 0, 10000, 0, 0,
                          self.bi,
                          self.bp],
                  100)
        P.plot(res)
        P.title('$SIRS_s$')
        P.legend(['E', 'I', 'S'])

    def test_run_SIRS(self):
        model = Epimodel(1, modtype=b'SIRS')
        res = run(model, [(0, 1, 999), 0, 10000, 0, 0,
                          self.bi,
                          self.bp],
                  100)
        P.plot(res)
        P.title('$SIRS$')
        P.legend(['E', 'I', 'S'])

    def test_run_SEIpR_s(self):
        model = Epimodel(1, modtype=b'SEIpR_s')
        res = run(model, [(0, 10, 990), 0, 10000, 0, 0,
                          self.bi,
                          self.bp],
                  100)
        P.plot(res)
        P.title('$SEIpR_s$')
        P.legend(['E', 'I', 'S'])

    def test_run_SEIpR(self):
        model = Epimodel(1, modtype=b'SEIpR')
        res = run(model, [(0, 10, 990), 0, 10000, 0, 0,
                          self.bi,
                          self.bp],
                  100)
        P.plot(res)
        P.title('$SEIpR$')
        P.legend(['E', 'I', 'S'])

    def test_run_SIpR_s(self):
        model = Epimodel(1, modtype=b'SIpR_s')
        res = run(model, [(0, 10, 990), 0, 10000, 0, 0,
                          self.bi,
                          self.bp],
                  100)
        P.plot(res)
        P.title('$SIpR_s$')
        P.legend(['E', 'I', 'S'])

    def test_run_SIpR(self):
        model = Epimodel(1, modtype=b'SIpR')
        res = run(model, [(0, 10, 990), 0, 10000, 0, 0,
                          self.bi,
                          self.bp],
                  100)
        P.plot(res)
        P.title('$SIpR$')
        P.legend(['E', 'I', 'S'])

    def test_run_SEIpRpS_s(self):
        model = Epimodel(1, modtype=b'SEIpRpS_s')
        res = run(model, [(0, 10, 990), 0, 10000, 0, 0,
                          self.bi,
                          self.bp],
                  100)
        P.plot(res)
        P.title('$SEIpRpS_s$')
        P.legend(['E', 'I', 'S'])

    def test_run_SEIpRpS(self):
        model = Epimodel(1, modtype=b'SEIpRpS')
        res = run(model, [(0, 10, 990), 0, 10000, 0, 0,
                          self.bi,
                          self.bp],
                  100)
        P.plot(res)
        P.title('$SEIpRpS$')
        P.legend(['E', 'I', 'S'])

    def test_run_SIpRpS_s(self):
        model = Epimodel(1, modtype=b'SIpRpS_s')
        res = run(model, [(0, 10, 990), 0, 10000, 0, 0,
                          self.bi,
                          self.bp],
                  100)
        P.plot(res)
        P.title('$SIpRpS_s$')
        P.legend(['E', 'I', 'S'])

    def test_run_SIpRpS(self):
        model = Epimodel(1, modtype=b'SIpRpS')
        res = run(model, [(0, 10, 990), 0, 10000, 0, 0,
                          self.bi,
                          self.bp],
                  100)
        P.plot(res)
        P.title('$SIpRpS$')
        P.legend(['E', 'I', 'S'])

    def test_run_SEIR_s(self):
        model = Epimodel(1, modtype=b'SEIR_s')
        res = run(model, [(0, 10, 990), 0, 10000, 0, 0,
                          self.bi,
                          self.bp],
                  100)
        P.plot(res)
        P.title('$SEIR_s$')
        P.legend(['E', 'I', 'S'])

    def test_run_SEIR(self):
        model = Epimodel(1, modtype=b'SEIR')
        res = run(model, [(0, 10, 990), 0, 10000, 0, 0,
                          self.bi,
                          self.bp],
                  100)
        P.plot(res)
        P.title('$SEIR$')
        P.legend(['E', 'I', 'S'])

    def test_run_SEIS_s(self):
        model = Epimodel(1, modtype=b'SEIS_s')
        res = run(model, [(0, 10, 990), 0, 10000, 0, 0,
                          self.bi,
                          self.bp],
                  100)
        P.plot(res)
        P.title('$SEIS_s$')
        P.legend(['E', 'I', 'S'])

    def test_run_SEIS(self):
        model = Epimodel(1, modtype=b'SEIS')
        res = run(model, [(0, 10, 990), 0, 10000, 0, 0,
                          self.bi,
                          self.bp],
                  100)
        P.plot(res)
        P.title('$SEIS$')
        P.legend(['E', 'I', 'S'])

    def test_run_SIR_s(self):
        model = Epimodel(1, modtype=b'SIR_s')
        res = run(model, [(0, 10, 990), 0, 10000, 0, 0,
                          self.bi,
                          self.bp],
                  100)
        P.plot(res)
        P.title('$SIR_s$')
        P.legend(['E', 'I', 'S'])

    def test_run_SIR(self):
        model = Epimodel(1, modtype=b'SIR')
        res = run(model, [(0, 10, 990), 0, 10000, 0, 0,
                          self.bi,
                          self.bp],
                  100)
        P.plot(res)
        P.title('$SIR$')
        P.legend(['E', 'I', 'S'])

    def test_run_SIS_s(self):
        model = Epimodel(1, modtype=b'SIS_s')
        res = run(model, [(0, 10, 990), 0, 10000, 0, 0,
                          self.bi,
                          self.bp],
                  100)
        P.plot(res)
        P.title('$SIS_s$')
        P.legend(['E', 'I', 'S'])

    def test_run_SIS(self):
        model = Epimodel(1, modtype=b'SIS')
        res = run(model, [(0, 10, 990), 0, 10000, 0, 0,
                          self.bi,
                          self.bp],
                  100)
        P.plot(res)
        P.title('$SIS$')
        P.legend(['E', 'I', 'S'])



    def test_run_Influenza(self):
        model = Epimodel(1, modtype=b'Influenza')
        res = run(model, [(2490, 0, 10, 0, 0, 2490, 0, 10, 0, 0, 2490, 0, 10, 0, 0, 2490, 0, 10, 0, 0), 0, 10000, 0, 0,
                          {b'incub_age1': 0, b'subc_age1': 10, b'susc_age1': 2490, b'incub_age2': 0, b'subc_age2': 10, b'susc_age2': 2490, b'incub_age3': 0, b'subc_age3': 10,
                           b'susc_age3': 2490,
                           b'incub_age4': 0, b'subc_age4': 10, b'susc_age4': 2490, b'sympt_age1': 0, b'sympt_age2': 0, b'sympt_age3': 0, b'sympt_age4': 0, b'comp_age1': 0,
                           b'comp_age2': 0,
                           b'comp_age3': 0, b'comp_age4': 0},
                          {b'r': 0.1, b'b': 0.01, b'beta': 1, b'e': 0.1, b'pc1': .1, b'c': 0.1, b'pp1': .1,
                           b'g': .1,
                           b'd': .1, b'pc2': .1, b'pp2': .1, b'pc3': .1, b'pp3': .1, b'pc4': .1, b'pp4': .1,
                           b'alpha': 1., b'delta': 0.1, b'vaccineNow': 1, b'vaccov': 0.3}], 100)
        P.plot(res)
        P.title('Influenza')
        P.legend(['Susc_age1', 'Incub_age1', 'Subc_age1', 'Sympt_age1', 'Comp_age1',
                  'Susc_age2', 'Incub_age2', 'Subc_age2', 'Sympt_age2', 'Comp_age2',
                  'Susc_age3', 'Incub_age3', 'Subc_age3', 'Sympt_age3', 'Comp_age3',
                  'Susc_age4', 'Incub_age4', 'Subc_age4', 'Sympt_age4', 'Comp_age4'])